# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Mapping, Optional, cast
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import (
    memory_add_params,
    memory_list_params,
    memory_search_params,
    memory_update_params,
    memory_upload_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from .._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.memory import Memory
from ..types.memory_status import MemoryStatus
from ..types.shared.query_result import QueryResult
from ..types.memory_delete_response import MemoryDeleteResponse
from ..types.memory_status_response import MemoryStatusResponse

__all__ = ["MemoriesResource", "AsyncMemoriesResource"]


class MemoriesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MemoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return MemoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MemoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return MemoriesResourceWithStreamingResponse(self)

    def update(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        collection: Union[str, object, None] | Omit = omit,
        metadata: Union[Dict[str, Union[str, float, bool]], object, None] | Omit = omit,
        text: Union[str, object, None] | Omit = omit,
        title: Union[str, object, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """Updates an existing document in the index.

        You can update the text, collection,
        title, and metadata. The document must already exist or a 404 will be returned.
        This works for documents from any source (vault, slack, gmail, etc.).

        To remove a collection, set it to null explicitly.

        Args:
          collection: The collection to move the document to. Set to null to remove the collection.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars. Values must be string, number, or boolean. Will be merged with
              existing metadata.

          text: Full text of the document. If provided, the document will be re-indexed.

          title: Title of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._post(
            f"/memories/update/{source}/{resource_id}",
            body=maybe_transform(
                {
                    "collection": collection,
                    "metadata": metadata,
                    "text": text,
                    "title": title,
                },
                memory_update_params.MemoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )

    def list(
        self,
        *,
        collection: Optional[str] | Omit = omit,
        cursor: Optional[str] | Omit = omit,
        filter: Optional[str] | Omit = omit,
        size: int | Omit = omit,
        source: Optional[
            Literal[
                "collections",
                "vault",
                "web_crawler",
                "notion",
                "slack",
                "google_calendar",
                "reddit",
                "box",
                "google_drive",
                "airtable",
                "algolia",
                "amplitude",
                "asana",
                "ashby",
                "bamboohr",
                "basecamp",
                "bubbles",
                "calendly",
                "confluence",
                "clickup",
                "datadog",
                "deel",
                "discord",
                "dropbox",
                "exa",
                "facebook",
                "front",
                "github",
                "gitlab",
                "google_docs",
                "google_mail",
                "google_sheet",
                "hubspot",
                "jira",
                "linear",
                "microsoft_teams",
                "mixpanel",
                "monday",
                "outlook",
                "perplexity",
                "rippling",
                "salesforce",
                "segment",
                "todoist",
                "twitter",
                "zoom",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[Memory]:
        """This endpoint allows you to paginate through all documents in the index.

        You can
        filter the documents by title, date, metadata, etc.

        Args:
          collection: Filter documents by collection.

          filter:
              Filter documents by metadata using MongoDB-style operators. Example:
              {"department": "engineering", "priority": {"$gt": 3}}

          source: Filter documents by source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/memories/list",
            page=SyncCursorPage[Memory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "collection": collection,
                        "cursor": cursor,
                        "filter": filter,
                        "size": size,
                        "source": source,
                    },
                    memory_list_params.MemoryListParams,
                ),
            ),
            model=Memory,
        )

    def delete(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryDeleteResponse:
        """
        Delete a memory and its associated chunks from the index.

        This removes the memory completely from the vector index and database. The
        operation deletes:

        1. All chunks associated with the resource (including embeddings)
        2. The resource record itself

        Args: source: The document provider (e.g., gmail, notion, vault) resource_id:
        The unique identifier of the resource to delete api_token: Authentication token

        Returns: MemoryDeletionResponse with deletion details

        Raises: DocumentNotFound: If the resource doesn't exist or user doesn't have
        access

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._delete(
            f"/memories/delete/{source}/{resource_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryDeleteResponse,
        )

    def add(
        self,
        *,
        text: str,
        collection: Optional[str] | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        resource_id: str | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """Adds an arbitrary document to the index.

        This can be any text, email, call
        transcript, etc. The document will be processed and made available for querying
        once the processing is complete.

        Args:
          text: Full text of the document.

          collection: The collection to add the document to for easier retrieval.

          date: Date of the document. Depending on the document, this could be the creation date
              or date the document was last updated (eg. for a chat transcript, this would be
              the date of the last message). This helps the ranking algorithm and allows you
              to filter by date range.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars. Values must be string, number, or boolean.

          resource_id: The resource ID to add the document to. If not provided, a new resource ID will
              be generated. If provided, the document will be updated if it already exists.

          title: Title of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/memories/add",
            body=maybe_transform(
                {
                    "text": text,
                    "collection": collection,
                    "date": date,
                    "metadata": metadata,
                    "resource_id": resource_id,
                    "title": title,
                },
                memory_add_params.MemoryAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )

    def get(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Memory:
        """
        Retrieves a document by provider and resource_id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._get(
            f"/memories/get/{source}/{resource_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Memory,
        )

    def search(
        self,
        *,
        query: str,
        answer: bool | Omit = omit,
        max_results: int | Omit = omit,
        options: memory_search_params.Options | Omit = omit,
        sources: List[
            Literal[
                "collections",
                "vault",
                "web_crawler",
                "notion",
                "slack",
                "google_calendar",
                "reddit",
                "box",
                "google_drive",
                "airtable",
                "algolia",
                "amplitude",
                "asana",
                "ashby",
                "bamboohr",
                "basecamp",
                "bubbles",
                "calendly",
                "confluence",
                "clickup",
                "datadog",
                "deel",
                "discord",
                "dropbox",
                "exa",
                "facebook",
                "front",
                "github",
                "gitlab",
                "google_docs",
                "google_mail",
                "google_sheet",
                "hubspot",
                "jira",
                "linear",
                "microsoft_teams",
                "mixpanel",
                "monday",
                "outlook",
                "perplexity",
                "rippling",
                "salesforce",
                "segment",
                "todoist",
                "twitter",
                "zoom",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueryResult:
        """
        Retrieves documents matching the query.

        Args:
          query: Query to run.

          answer: If true, the query will be answered along with matching source documents.

          max_results: Maximum number of results to return.

          options: Search options for the query.

          sources: Only query documents from these sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/memories/query",
            body=maybe_transform(
                {
                    "query": query,
                    "answer": answer,
                    "max_results": max_results,
                    "options": options,
                    "sources": sources,
                },
                memory_search_params.MemorySearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueryResult,
        )

    def status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatusResponse:
        """
        This endpoint shows the indexing progress of documents, both by provider and
        total.
        """
        return self._get(
            "/memories/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatusResponse,
        )

    def upload(
        self,
        *,
        file: FileTypes,
        collection: Optional[str] | Omit = omit,
        metadata: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """This endpoint will upload a file to the index and return a resource_id.

        The file
        will be processed in the background and the memory will be available for
        querying once the processing is complete. You can use the `resource_id` to query
        the memory later, and check the status of the memory.

        Args:
          file: The file to ingest.

          collection: The collection to add the document to.

          metadata: Custom metadata as JSON string for filtering. Keys must be alphanumeric with
              underscores, max 64 chars. Values must be string, number, or boolean.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "collection": collection,
                "metadata": metadata,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/memories/upload",
            body=maybe_transform(body, memory_upload_params.MemoryUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )


class AsyncMemoriesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMemoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMemoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMemoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncMemoriesResourceWithStreamingResponse(self)

    async def update(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        collection: Union[str, object, None] | Omit = omit,
        metadata: Union[Dict[str, Union[str, float, bool]], object, None] | Omit = omit,
        text: Union[str, object, None] | Omit = omit,
        title: Union[str, object, None] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """Updates an existing document in the index.

        You can update the text, collection,
        title, and metadata. The document must already exist or a 404 will be returned.
        This works for documents from any source (vault, slack, gmail, etc.).

        To remove a collection, set it to null explicitly.

        Args:
          collection: The collection to move the document to. Set to null to remove the collection.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars. Values must be string, number, or boolean. Will be merged with
              existing metadata.

          text: Full text of the document. If provided, the document will be re-indexed.

          title: Title of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._post(
            f"/memories/update/{source}/{resource_id}",
            body=await async_maybe_transform(
                {
                    "collection": collection,
                    "metadata": metadata,
                    "text": text,
                    "title": title,
                },
                memory_update_params.MemoryUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )

    def list(
        self,
        *,
        collection: Optional[str] | Omit = omit,
        cursor: Optional[str] | Omit = omit,
        filter: Optional[str] | Omit = omit,
        size: int | Omit = omit,
        source: Optional[
            Literal[
                "collections",
                "vault",
                "web_crawler",
                "notion",
                "slack",
                "google_calendar",
                "reddit",
                "box",
                "google_drive",
                "airtable",
                "algolia",
                "amplitude",
                "asana",
                "ashby",
                "bamboohr",
                "basecamp",
                "bubbles",
                "calendly",
                "confluence",
                "clickup",
                "datadog",
                "deel",
                "discord",
                "dropbox",
                "exa",
                "facebook",
                "front",
                "github",
                "gitlab",
                "google_docs",
                "google_mail",
                "google_sheet",
                "hubspot",
                "jira",
                "linear",
                "microsoft_teams",
                "mixpanel",
                "monday",
                "outlook",
                "perplexity",
                "rippling",
                "salesforce",
                "segment",
                "todoist",
                "twitter",
                "zoom",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Memory, AsyncCursorPage[Memory]]:
        """This endpoint allows you to paginate through all documents in the index.

        You can
        filter the documents by title, date, metadata, etc.

        Args:
          collection: Filter documents by collection.

          filter:
              Filter documents by metadata using MongoDB-style operators. Example:
              {"department": "engineering", "priority": {"$gt": 3}}

          source: Filter documents by source.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/memories/list",
            page=AsyncCursorPage[Memory],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "collection": collection,
                        "cursor": cursor,
                        "filter": filter,
                        "size": size,
                        "source": source,
                    },
                    memory_list_params.MemoryListParams,
                ),
            ),
            model=Memory,
        )

    async def delete(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryDeleteResponse:
        """
        Delete a memory and its associated chunks from the index.

        This removes the memory completely from the vector index and database. The
        operation deletes:

        1. All chunks associated with the resource (including embeddings)
        2. The resource record itself

        Args: source: The document provider (e.g., gmail, notion, vault) resource_id:
        The unique identifier of the resource to delete api_token: Authentication token

        Returns: MemoryDeletionResponse with deletion details

        Raises: DocumentNotFound: If the resource doesn't exist or user doesn't have
        access

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._delete(
            f"/memories/delete/{source}/{resource_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryDeleteResponse,
        )

    async def add(
        self,
        *,
        text: str,
        collection: Optional[str] | Omit = omit,
        date: Union[str, datetime] | Omit = omit,
        metadata: Optional[Dict[str, Union[str, float, bool]]] | Omit = omit,
        resource_id: str | Omit = omit,
        title: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """Adds an arbitrary document to the index.

        This can be any text, email, call
        transcript, etc. The document will be processed and made available for querying
        once the processing is complete.

        Args:
          text: Full text of the document.

          collection: The collection to add the document to for easier retrieval.

          date: Date of the document. Depending on the document, this could be the creation date
              or date the document was last updated (eg. for a chat transcript, this would be
              the date of the last message). This helps the ranking algorithm and allows you
              to filter by date range.

          metadata: Custom metadata for filtering. Keys must be alphanumeric with underscores, max
              64 chars. Values must be string, number, or boolean.

          resource_id: The resource ID to add the document to. If not provided, a new resource ID will
              be generated. If provided, the document will be updated if it already exists.

          title: Title of the document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/memories/add",
            body=await async_maybe_transform(
                {
                    "text": text,
                    "collection": collection,
                    "date": date,
                    "metadata": metadata,
                    "resource_id": resource_id,
                    "title": title,
                },
                memory_add_params.MemoryAddParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )

    async def get(
        self,
        resource_id: str,
        *,
        source: Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Memory:
        """
        Retrieves a document by provider and resource_id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not source:
            raise ValueError(f"Expected a non-empty value for `source` but received {source!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._get(
            f"/memories/get/{source}/{resource_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Memory,
        )

    async def search(
        self,
        *,
        query: str,
        answer: bool | Omit = omit,
        max_results: int | Omit = omit,
        options: memory_search_params.Options | Omit = omit,
        sources: List[
            Literal[
                "collections",
                "vault",
                "web_crawler",
                "notion",
                "slack",
                "google_calendar",
                "reddit",
                "box",
                "google_drive",
                "airtable",
                "algolia",
                "amplitude",
                "asana",
                "ashby",
                "bamboohr",
                "basecamp",
                "bubbles",
                "calendly",
                "confluence",
                "clickup",
                "datadog",
                "deel",
                "discord",
                "dropbox",
                "exa",
                "facebook",
                "front",
                "github",
                "gitlab",
                "google_docs",
                "google_mail",
                "google_sheet",
                "hubspot",
                "jira",
                "linear",
                "microsoft_teams",
                "mixpanel",
                "monday",
                "outlook",
                "perplexity",
                "rippling",
                "salesforce",
                "segment",
                "todoist",
                "twitter",
                "zoom",
            ]
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueryResult:
        """
        Retrieves documents matching the query.

        Args:
          query: Query to run.

          answer: If true, the query will be answered along with matching source documents.

          max_results: Maximum number of results to return.

          options: Search options for the query.

          sources: Only query documents from these sources.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/memories/query",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "answer": answer,
                    "max_results": max_results,
                    "options": options,
                    "sources": sources,
                },
                memory_search_params.MemorySearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueryResult,
        )

    async def status(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatusResponse:
        """
        This endpoint shows the indexing progress of documents, both by provider and
        total.
        """
        return await self._get(
            "/memories/status",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatusResponse,
        )

    async def upload(
        self,
        *,
        file: FileTypes,
        collection: Optional[str] | Omit = omit,
        metadata: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemoryStatus:
        """This endpoint will upload a file to the index and return a resource_id.

        The file
        will be processed in the background and the memory will be available for
        querying once the processing is complete. You can use the `resource_id` to query
        the memory later, and check the status of the memory.

        Args:
          file: The file to ingest.

          collection: The collection to add the document to.

          metadata: Custom metadata as JSON string for filtering. Keys must be alphanumeric with
              underscores, max 64 chars. Values must be string, number, or boolean.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "collection": collection,
                "metadata": metadata,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/memories/upload",
            body=await async_maybe_transform(body, memory_upload_params.MemoryUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemoryStatus,
        )


class MemoriesResourceWithRawResponse:
    def __init__(self, memories: MemoriesResource) -> None:
        self._memories = memories

        self.update = to_raw_response_wrapper(
            memories.update,
        )
        self.list = to_raw_response_wrapper(
            memories.list,
        )
        self.delete = to_raw_response_wrapper(
            memories.delete,
        )
        self.add = to_raw_response_wrapper(
            memories.add,
        )
        self.get = to_raw_response_wrapper(
            memories.get,
        )
        self.search = to_raw_response_wrapper(
            memories.search,
        )
        self.status = to_raw_response_wrapper(
            memories.status,
        )
        self.upload = to_raw_response_wrapper(
            memories.upload,
        )


class AsyncMemoriesResourceWithRawResponse:
    def __init__(self, memories: AsyncMemoriesResource) -> None:
        self._memories = memories

        self.update = async_to_raw_response_wrapper(
            memories.update,
        )
        self.list = async_to_raw_response_wrapper(
            memories.list,
        )
        self.delete = async_to_raw_response_wrapper(
            memories.delete,
        )
        self.add = async_to_raw_response_wrapper(
            memories.add,
        )
        self.get = async_to_raw_response_wrapper(
            memories.get,
        )
        self.search = async_to_raw_response_wrapper(
            memories.search,
        )
        self.status = async_to_raw_response_wrapper(
            memories.status,
        )
        self.upload = async_to_raw_response_wrapper(
            memories.upload,
        )


class MemoriesResourceWithStreamingResponse:
    def __init__(self, memories: MemoriesResource) -> None:
        self._memories = memories

        self.update = to_streamed_response_wrapper(
            memories.update,
        )
        self.list = to_streamed_response_wrapper(
            memories.list,
        )
        self.delete = to_streamed_response_wrapper(
            memories.delete,
        )
        self.add = to_streamed_response_wrapper(
            memories.add,
        )
        self.get = to_streamed_response_wrapper(
            memories.get,
        )
        self.search = to_streamed_response_wrapper(
            memories.search,
        )
        self.status = to_streamed_response_wrapper(
            memories.status,
        )
        self.upload = to_streamed_response_wrapper(
            memories.upload,
        )


class AsyncMemoriesResourceWithStreamingResponse:
    def __init__(self, memories: AsyncMemoriesResource) -> None:
        self._memories = memories

        self.update = async_to_streamed_response_wrapper(
            memories.update,
        )
        self.list = async_to_streamed_response_wrapper(
            memories.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            memories.delete,
        )
        self.add = async_to_streamed_response_wrapper(
            memories.add,
        )
        self.get = async_to_streamed_response_wrapper(
            memories.get,
        )
        self.search = async_to_streamed_response_wrapper(
            memories.search,
        )
        self.status = async_to_streamed_response_wrapper(
            memories.status,
        )
        self.upload = async_to_streamed_response_wrapper(
            memories.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Memory", "Metadata", "MetadataEvent"]


class MetadataEvent(BaseModel):
    message: str

    type: Literal["error", "warning", "info", "success"]

    time: Optional[datetime] = None


class Metadata(BaseModel):
    created_at: Optional[datetime] = None

    events: Optional[List[MetadataEvent]] = None

    indexed_at: Optional[datetime] = None

    last_modified: Optional[datetime] = None

    status: Optional[Literal["pending", "processing", "completed", "failed"]] = None

    url: Optional[str] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class Memory(BaseModel):
    resource_id: str

    source: Literal[
        "collections",
        "vault",
        "web_crawler",
        "notion",
        "slack",
        "google_calendar",
        "reddit",
        "box",
        "google_drive",
        "airtable",
        "algolia",
        "amplitude",
        "asana",
        "ashby",
        "bamboohr",
        "basecamp",
        "bubbles",
        "calendly",
        "confluence",
        "clickup",
        "datadog",
        "deel",
        "discord",
        "dropbox",
        "exa",
        "facebook",
        "front",
        "github",
        "gitlab",
        "google_docs",
        "google_mail",
        "google_sheet",
        "hubspot",
        "jira",
        "linear",
        "microsoft_teams",
        "mixpanel",
        "monday",
        "outlook",
        "perplexity",
        "rippling",
        "salesforce",
        "segment",
        "todoist",
        "twitter",
        "zoom",
    ]

    metadata: Optional[Metadata] = None

    score: Optional[float] = None
    """The relevance of the resource to the query"""

    title: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["EvaluateScoreHighlightParams"]


class EvaluateScoreHighlightParams(TypedDict, total=False):
    comment: Optional[str]
    """Comment on the chunk"""

    score: float
    """Rating of the chunk from -1 (bad) to +1 (good)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.integrations import slack_list_params

__all__ = ["SlackResource", "AsyncSlackResource"]


class SlackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SlackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return SlackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SlackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return SlackResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        channels: SequenceNotStr[str] | Omit = omit,
        exclude_archived: Optional[bool] | Omit = omit,
        include_dms: bool | Omit = omit,
        include_group_dms: bool | Omit = omit,
        include_private: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List Slack conversations accessible to the user via the live Nango connection.

        Returns minimal channel metadata suitable for selection UIs. If required scopes
        are missing, Slack's error is propagated with details.

        Supports filtering by channels, including/excluding private channels, DMs, group
        DMs, and archived channels based on the provided search options.

        Args:
          channels: List of Slack channels to include (by id, name, or #name).

          exclude_archived: If set, pass 'exclude_archived' to Slack. If None, omit the param.

          include_dms: Include direct messages (im) when listing conversations.

          include_group_dms: Include group DMs (mpim) when listing conversations.

          include_private: Include private channels when constructing Slack 'types'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/integrations/slack/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "channels": channels,
                        "exclude_archived": exclude_archived,
                        "include_dms": include_dms,
                        "include_group_dms": include_group_dms,
                        "include_private": include_private,
                    },
                    slack_list_params.SlackListParams,
                ),
            ),
            cast_to=object,
        )


class AsyncSlackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSlackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncSlackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSlackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncSlackResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        channels: SequenceNotStr[str] | Omit = omit,
        exclude_archived: Optional[bool] | Omit = omit,
        include_dms: bool | Omit = omit,
        include_group_dms: bool | Omit = omit,
        include_private: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        List Slack conversations accessible to the user via the live Nango connection.

        Returns minimal channel metadata suitable for selection UIs. If required scopes
        are missing, Slack's error is propagated with details.

        Supports filtering by channels, including/excluding private channels, DMs, group
        DMs, and archived channels based on the provided search options.

        Args:
          channels: List of Slack channels to include (by id, name, or #name).

          exclude_archived: If set, pass 'exclude_archived' to Slack. If None, omit the param.

          include_dms: Include direct messages (im) when listing conversations.

          include_group_dms: Include group DMs (mpim) when listing conversations.

          include_private: Include private channels when constructing Slack 'types'.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/integrations/slack/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "channels": channels,
                        "exclude_archived": exclude_archived,
                        "include_dms": include_dms,
                        "include_group_dms": include_group_dms,
                        "include_private": include_private,
                    },
                    slack_list_params.SlackListParams,
                ),
            ),
            cast_to=object,
        )


class SlackResourceWithRawResponse:
    def __init__(self, slack: SlackResource) -> None:
        self._slack = slack

        self.list = to_raw_response_wrapper(
            slack.list,
        )


class AsyncSlackResourceWithRawResponse:
    def __init__(self, slack: AsyncSlackResource) -> None:
        self._slack = slack

        self.list = async_to_raw_response_wrapper(
            slack.list,
        )


class SlackResourceWithStreamingResponse:
    def __init__(self, slack: SlackResource) -> None:
        self._slack = slack

        self.list = to_streamed_response_wrapper(
            slack.list,
        )


class AsyncSlackResourceWithStreamingResponse:
    def __init__(self, slack: AsyncSlackResource) -> None:
        self._slack = slack

        self.list = async_to_streamed_response_wrapper(
            slack.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["AuthUserTokenParams"]


class AuthUserTokenParams(TypedDict, total=False):
    user_id: Required[str]

    expires_in: Optional[str]
    """Token lifetime, e.g., '30m', '2h', '1d'. Defaults to 24 hours if not provided."""

    origin: Optional[str]
    """Origin of the request, used for CSRF protection.

    If set, the token will only be valid for requests originating from this origin.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["MemoryListParams"]


class MemoryListParams(TypedDict, total=False):
    collection: Optional[str]
    """Filter documents by collection."""

    cursor: Optional[str]

    filter: Optional[str]
    """Filter documents by metadata using MongoDB-style operators.

    Example: {"department": "engineering", "priority": {"$gt": 3}}
    """

    size: int

    source: Optional[
        Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ]
    ]
    """Filter documents by source."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .slack import (
    SlackResource,
    AsyncSlackResource,
    SlackResourceWithRawResponse,
    AsyncSlackResourceWithRawResponse,
    SlackResourceWithStreamingResponse,
    AsyncSlackResourceWithStreamingResponse,
)
from .web_crawler import (
    WebCrawlerResource,
    AsyncWebCrawlerResource,
    WebCrawlerResourceWithRawResponse,
    AsyncWebCrawlerResourceWithRawResponse,
    WebCrawlerResourceWithStreamingResponse,
    AsyncWebCrawlerResourceWithStreamingResponse,
)
from .integrations import (
    IntegrationsResource,
    AsyncIntegrationsResource,
    IntegrationsResourceWithRawResponse,
    AsyncIntegrationsResourceWithRawResponse,
    IntegrationsResourceWithStreamingResponse,
    AsyncIntegrationsResourceWithStreamingResponse,
)
from .google_calendar import (
    GoogleCalendarResource,
    AsyncGoogleCalendarResource,
    GoogleCalendarResourceWithRawResponse,
    AsyncGoogleCalendarResourceWithRawResponse,
    GoogleCalendarResourceWithStreamingResponse,
    AsyncGoogleCalendarResourceWithStreamingResponse,
)

__all__ = [
    "GoogleCalendarResource",
    "AsyncGoogleCalendarResource",
    "GoogleCalendarResourceWithRawResponse",
    "AsyncGoogleCalendarResourceWithRawResponse",
    "GoogleCalendarResourceWithStreamingResponse",
    "AsyncGoogleCalendarResourceWithStreamingResponse",
    "WebCrawlerResource",
    "AsyncWebCrawlerResource",
    "WebCrawlerResourceWithRawResponse",
    "AsyncWebCrawlerResourceWithRawResponse",
    "WebCrawlerResourceWithStreamingResponse",
    "AsyncWebCrawlerResourceWithStreamingResponse",
    "SlackResource",
    "AsyncSlackResource",
    "SlackResourceWithRawResponse",
    "AsyncSlackResourceWithRawResponse",
    "SlackResourceWithStreamingResponse",
    "AsyncSlackResourceWithStreamingResponse",
    "IntegrationsResource",
    "AsyncIntegrationsResource",
    "IntegrationsResourceWithRawResponse",
    "AsyncIntegrationsResourceWithRawResponse",
    "IntegrationsResourceWithStreamingResponse",
    "AsyncIntegrationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["VaultListResponse"]


class VaultListResponse(BaseModel):
    collection: Optional[str] = None

    document_count: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AuthMeResponse", "App"]


class App(BaseModel):
    """The Hyperspell app's id this user belongs to"""

    id: str
    """The Hyperspell app's id this user belongs to"""

    icon_url: Optional[str] = None
    """The app's icon"""

    name: str
    """The app's name"""

    redirect_url: Optional[str] = None
    """The app's redirect URL"""


class AuthMeResponse(BaseModel):
    id: str
    """The user's id"""

    app: App
    """The Hyperspell app's id this user belongs to"""

    available_integrations: List[
        Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ]
    ]
    """All integrations available for the app"""

    installed_integrations: List[
        Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ]
    ]
    """All integrations installed for the user"""

    token_expiration: Optional[datetime] = None
    """The expiration time of the user token used to make the request"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import evaluate_score_query_params, evaluate_score_highlight_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.shared.query_result import QueryResult
from ..types.evaluate_score_query_response import EvaluateScoreQueryResponse
from ..types.evaluate_score_highlight_response import EvaluateScoreHighlightResponse

__all__ = ["EvaluateResource", "AsyncEvaluateResource"]


class EvaluateResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return EvaluateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return EvaluateResourceWithStreamingResponse(self)

    def get_query(
        self,
        query_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueryResult:
        """
        Retrieve the result of a previous query.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return self._get(
            f"/evaluate/query/{query_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueryResult,
        )

    def score_highlight(
        self,
        highlight_id: str,
        *,
        comment: Optional[str] | Omit = omit,
        score: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateScoreHighlightResponse:
        """
        Score an individual highlight.

        Args:
          comment: Comment on the chunk

          score: Rating of the chunk from -1 (bad) to +1 (good).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not highlight_id:
            raise ValueError(f"Expected a non-empty value for `highlight_id` but received {highlight_id!r}")
        return self._post(
            f"/evaluate/highlight/{highlight_id}",
            body=maybe_transform(
                {
                    "comment": comment,
                    "score": score,
                },
                evaluate_score_highlight_params.EvaluateScoreHighlightParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluateScoreHighlightResponse,
        )

    def score_query(
        self,
        query_id: str,
        *,
        score: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateScoreQueryResponse:
        """
        Score the result of a query.

        Args:
          score: Rating of the query result from -1 (bad) to +1 (good).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return self._post(
            f"/evaluate/query/{query_id}",
            body=maybe_transform({"score": score}, evaluate_score_query_params.EvaluateScoreQueryParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluateScoreQueryResponse,
        )


class AsyncEvaluateResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluateResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluateResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluateResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncEvaluateResourceWithStreamingResponse(self)

    async def get_query(
        self,
        query_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueryResult:
        """
        Retrieve the result of a previous query.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return await self._get(
            f"/evaluate/query/{query_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueryResult,
        )

    async def score_highlight(
        self,
        highlight_id: str,
        *,
        comment: Optional[str] | Omit = omit,
        score: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateScoreHighlightResponse:
        """
        Score an individual highlight.

        Args:
          comment: Comment on the chunk

          score: Rating of the chunk from -1 (bad) to +1 (good).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not highlight_id:
            raise ValueError(f"Expected a non-empty value for `highlight_id` but received {highlight_id!r}")
        return await self._post(
            f"/evaluate/highlight/{highlight_id}",
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "score": score,
                },
                evaluate_score_highlight_params.EvaluateScoreHighlightParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluateScoreHighlightResponse,
        )

    async def score_query(
        self,
        query_id: str,
        *,
        score: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluateScoreQueryResponse:
        """
        Score the result of a query.

        Args:
          score: Rating of the query result from -1 (bad) to +1 (good).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not query_id:
            raise ValueError(f"Expected a non-empty value for `query_id` but received {query_id!r}")
        return await self._post(
            f"/evaluate/query/{query_id}",
            body=await async_maybe_transform({"score": score}, evaluate_score_query_params.EvaluateScoreQueryParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EvaluateScoreQueryResponse,
        )


class EvaluateResourceWithRawResponse:
    def __init__(self, evaluate: EvaluateResource) -> None:
        self._evaluate = evaluate

        self.get_query = to_raw_response_wrapper(
            evaluate.get_query,
        )
        self.score_highlight = to_raw_response_wrapper(
            evaluate.score_highlight,
        )
        self.score_query = to_raw_response_wrapper(
            evaluate.score_query,
        )


class AsyncEvaluateResourceWithRawResponse:
    def __init__(self, evaluate: AsyncEvaluateResource) -> None:
        self._evaluate = evaluate

        self.get_query = async_to_raw_response_wrapper(
            evaluate.get_query,
        )
        self.score_highlight = async_to_raw_response_wrapper(
            evaluate.score_highlight,
        )
        self.score_query = async_to_raw_response_wrapper(
            evaluate.score_query,
        )


class EvaluateResourceWithStreamingResponse:
    def __init__(self, evaluate: EvaluateResource) -> None:
        self._evaluate = evaluate

        self.get_query = to_streamed_response_wrapper(
            evaluate.get_query,
        )
        self.score_highlight = to_streamed_response_wrapper(
            evaluate.score_highlight,
        )
        self.score_query = to_streamed_response_wrapper(
            evaluate.score_query,
        )


class AsyncEvaluateResourceWithStreamingResponse:
    def __init__(self, evaluate: AsyncEvaluateResource) -> None:
        self._evaluate = evaluate

        self.get_query = async_to_streamed_response_wrapper(
            evaluate.get_query,
        )
        self.score_highlight = async_to_streamed_response_wrapper(
            evaluate.score_highlight,
        )
        self.score_query = async_to_streamed_response_wrapper(
            evaluate.score_query,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["WebCrawlerIndexParams"]


class WebCrawlerIndexParams(TypedDict, total=False):
    url: Required[str]
    """The base URL of the website to crawl"""

    limit: int
    """Maximum number of pages to crawl in total"""

    max_depth: int
    """Maximum depth of links to follow during crawling"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from .slack import (
    SlackResource,
    AsyncSlackResource,
    SlackResourceWithRawResponse,
    AsyncSlackResourceWithRawResponse,
    SlackResourceWithStreamingResponse,
    AsyncSlackResourceWithStreamingResponse,
)
from ...types import integration_connect_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .web_crawler import (
    WebCrawlerResource,
    AsyncWebCrawlerResource,
    WebCrawlerResourceWithRawResponse,
    AsyncWebCrawlerResourceWithRawResponse,
    WebCrawlerResourceWithStreamingResponse,
    AsyncWebCrawlerResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .google_calendar import (
    GoogleCalendarResource,
    AsyncGoogleCalendarResource,
    GoogleCalendarResourceWithRawResponse,
    AsyncGoogleCalendarResourceWithRawResponse,
    GoogleCalendarResourceWithStreamingResponse,
    AsyncGoogleCalendarResourceWithStreamingResponse,
)
from ...types.integration_list_response import IntegrationListResponse
from ...types.integration_connect_response import IntegrationConnectResponse

__all__ = ["IntegrationsResource", "AsyncIntegrationsResource"]


class IntegrationsResource(SyncAPIResource):
    @cached_property
    def google_calendar(self) -> GoogleCalendarResource:
        return GoogleCalendarResource(self._client)

    @cached_property
    def web_crawler(self) -> WebCrawlerResource:
        return WebCrawlerResource(self._client)

    @cached_property
    def slack(self) -> SlackResource:
        return SlackResource(self._client)

    @cached_property
    def with_raw_response(self) -> IntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return IntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return IntegrationsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationListResponse:
        """List all integrations for the user."""
        return self._get(
            "/integrations/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationListResponse,
        )

    def connect(
        self,
        integration_id: str,
        *,
        redirect_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationConnectResponse:
        """
        Redirects to the connect URL to link an integration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not integration_id:
            raise ValueError(f"Expected a non-empty value for `integration_id` but received {integration_id!r}")
        return self._get(
            f"/integrations/{integration_id}/connect",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"redirect_url": redirect_url}, integration_connect_params.IntegrationConnectParams
                ),
            ),
            cast_to=IntegrationConnectResponse,
        )


class AsyncIntegrationsResource(AsyncAPIResource):
    @cached_property
    def google_calendar(self) -> AsyncGoogleCalendarResource:
        return AsyncGoogleCalendarResource(self._client)

    @cached_property
    def web_crawler(self) -> AsyncWebCrawlerResource:
        return AsyncWebCrawlerResource(self._client)

    @cached_property
    def slack(self) -> AsyncSlackResource:
        return AsyncSlackResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncIntegrationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncIntegrationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIntegrationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncIntegrationsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationListResponse:
        """List all integrations for the user."""
        return await self._get(
            "/integrations/list",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=IntegrationListResponse,
        )

    async def connect(
        self,
        integration_id: str,
        *,
        redirect_url: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> IntegrationConnectResponse:
        """
        Redirects to the connect URL to link an integration.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not integration_id:
            raise ValueError(f"Expected a non-empty value for `integration_id` but received {integration_id!r}")
        return await self._get(
            f"/integrations/{integration_id}/connect",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"redirect_url": redirect_url}, integration_connect_params.IntegrationConnectParams
                ),
            ),
            cast_to=IntegrationConnectResponse,
        )


class IntegrationsResourceWithRawResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.list = to_raw_response_wrapper(
            integrations.list,
        )
        self.connect = to_raw_response_wrapper(
            integrations.connect,
        )

    @cached_property
    def google_calendar(self) -> GoogleCalendarResourceWithRawResponse:
        return GoogleCalendarResourceWithRawResponse(self._integrations.google_calendar)

    @cached_property
    def web_crawler(self) -> WebCrawlerResourceWithRawResponse:
        return WebCrawlerResourceWithRawResponse(self._integrations.web_crawler)

    @cached_property
    def slack(self) -> SlackResourceWithRawResponse:
        return SlackResourceWithRawResponse(self._integrations.slack)


class AsyncIntegrationsResourceWithRawResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.list = async_to_raw_response_wrapper(
            integrations.list,
        )
        self.connect = async_to_raw_response_wrapper(
            integrations.connect,
        )

    @cached_property
    def google_calendar(self) -> AsyncGoogleCalendarResourceWithRawResponse:
        return AsyncGoogleCalendarResourceWithRawResponse(self._integrations.google_calendar)

    @cached_property
    def web_crawler(self) -> AsyncWebCrawlerResourceWithRawResponse:
        return AsyncWebCrawlerResourceWithRawResponse(self._integrations.web_crawler)

    @cached_property
    def slack(self) -> AsyncSlackResourceWithRawResponse:
        return AsyncSlackResourceWithRawResponse(self._integrations.slack)


class IntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: IntegrationsResource) -> None:
        self._integrations = integrations

        self.list = to_streamed_response_wrapper(
            integrations.list,
        )
        self.connect = to_streamed_response_wrapper(
            integrations.connect,
        )

    @cached_property
    def google_calendar(self) -> GoogleCalendarResourceWithStreamingResponse:
        return GoogleCalendarResourceWithStreamingResponse(self._integrations.google_calendar)

    @cached_property
    def web_crawler(self) -> WebCrawlerResourceWithStreamingResponse:
        return WebCrawlerResourceWithStreamingResponse(self._integrations.web_crawler)

    @cached_property
    def slack(self) -> SlackResourceWithStreamingResponse:
        return SlackResourceWithStreamingResponse(self._integrations.slack)


class AsyncIntegrationsResourceWithStreamingResponse:
    def __init__(self, integrations: AsyncIntegrationsResource) -> None:
        self._integrations = integrations

        self.list = async_to_streamed_response_wrapper(
            integrations.list,
        )
        self.connect = async_to_streamed_response_wrapper(
            integrations.connect,
        )

    @cached_property
    def google_calendar(self) -> AsyncGoogleCalendarResourceWithStreamingResponse:
        return AsyncGoogleCalendarResourceWithStreamingResponse(self._integrations.google_calendar)

    @cached_property
    def web_crawler(self) -> AsyncWebCrawlerResourceWithStreamingResponse:
        return AsyncWebCrawlerResourceWithStreamingResponse(self._integrations.web_crawler)

    @cached_property
    def slack(self) -> AsyncSlackResourceWithStreamingResponse:
        return AsyncSlackResourceWithStreamingResponse(self._integrations.slack)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from ..._types import SequenceNotStr

__all__ = ["SlackListParams"]


class SlackListParams(TypedDict, total=False):
    channels: SequenceNotStr[str]
    """List of Slack channels to include (by id, name, or #name)."""

    exclude_archived: Optional[bool]
    """If set, pass 'exclude_archived' to Slack. If None, omit the param."""

    include_dms: bool
    """Include direct messages (im) when listing conversations."""

    include_group_dms: bool
    """Include group DMs (mpim) when listing conversations."""

    include_private: bool
    """Include private channels when constructing Slack 'types'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncCursorPage", "AsyncCursorPage"]

_T = TypeVar("_T")


class SyncCursorPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    next_cursor: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_cursor = self.next_cursor
        if not next_cursor:
            return None

        return PageInfo(params={"cursor": next_cursor})


class AsyncCursorPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]
    next_cursor: Optional[str] = None

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_cursor = self.next_cursor
        if not next_cursor:
            return None

        return PageInfo(params={"cursor": next_cursor})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .token import Token as Token
from .memory import Memory as Memory
from .shared import QueryResult as QueryResult
from .memory_status import MemoryStatus as MemoryStatus
from .auth_me_response import AuthMeResponse as AuthMeResponse
from .memory_add_params import MemoryAddParams as MemoryAddParams
from .vault_list_params import VaultListParams as VaultListParams
from .memory_list_params import MemoryListParams as MemoryListParams
from .vault_list_response import VaultListResponse as VaultListResponse
from .memory_search_params import MemorySearchParams as MemorySearchParams
from .memory_update_params import MemoryUpdateParams as MemoryUpdateParams
from .memory_upload_params import MemoryUploadParams as MemoryUploadParams
from .auth_user_token_params import AuthUserTokenParams as AuthUserTokenParams
from .memory_delete_response import MemoryDeleteResponse as MemoryDeleteResponse
from .memory_status_response import MemoryStatusResponse as MemoryStatusResponse
from .connection_list_response import ConnectionListResponse as ConnectionListResponse
from .auth_delete_user_response import AuthDeleteUserResponse as AuthDeleteUserResponse
from .integration_list_response import IntegrationListResponse as IntegrationListResponse
from .connection_revoke_response import ConnectionRevokeResponse as ConnectionRevokeResponse
from .integration_connect_params import IntegrationConnectParams as IntegrationConnectParams
from .evaluate_score_query_params import EvaluateScoreQueryParams as EvaluateScoreQueryParams
from .integration_connect_response import IntegrationConnectResponse as IntegrationConnectResponse
from .evaluate_score_query_response import EvaluateScoreQueryResponse as EvaluateScoreQueryResponse
from .evaluate_score_highlight_params import EvaluateScoreHighlightParams as EvaluateScoreHighlightParams
from .evaluate_score_highlight_response import EvaluateScoreHighlightResponse as EvaluateScoreHighlightResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypedDict

__all__ = ["MemoryUpdateParams"]


class MemoryUpdateParams(TypedDict, total=False):
    source: Required[
        Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ]
    ]

    collection: Union[str, object, None]
    """The collection to move the document to. Set to null to remove the collection."""

    metadata: Union[Dict[str, Union[str, float, bool]], object, None]
    """Custom metadata for filtering.

    Keys must be alphanumeric with underscores, max 64 chars. Values must be string,
    number, or boolean. Will be merged with existing metadata.
    """

    text: Union[str, object, None]
    """Full text of the document. If provided, the document will be re-indexed."""

    title: Union[str, object, None]
    """Title of the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import auth_user_token_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.token import Token
from .._base_client import make_request_options
from ..types.auth_me_response import AuthMeResponse
from ..types.auth_delete_user_response import AuthDeleteUserResponse

__all__ = ["AuthResource", "AsyncAuthResource"]


class AuthResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AuthResourceWithStreamingResponse(self)

    def delete_user(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthDeleteUserResponse:
        """Endpoint to delete user."""
        return self._delete(
            "/auth/delete",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthDeleteUserResponse,
        )

    def me(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthMeResponse:
        """Endpoint to get basic user data."""
        return self._get(
            "/auth/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthMeResponse,
        )

    def user_token(
        self,
        *,
        user_id: str,
        expires_in: Optional[str] | Omit = omit,
        origin: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Token:
        """Use this endpoint to create a user token for a specific user.

        This token can be
        safely passed to your user-facing front-end.

        Args:
          expires_in: Token lifetime, e.g., '30m', '2h', '1d'. Defaults to 24 hours if not provided.

          origin: Origin of the request, used for CSRF protection. If set, the token will only be
              valid for requests originating from this origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/auth/user_token",
            body=maybe_transform(
                {
                    "user_id": user_id,
                    "expires_in": expires_in,
                    "origin": origin,
                },
                auth_user_token_params.AuthUserTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Token,
        )


class AsyncAuthResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncAuthResourceWithStreamingResponse(self)

    async def delete_user(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthDeleteUserResponse:
        """Endpoint to delete user."""
        return await self._delete(
            "/auth/delete",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthDeleteUserResponse,
        )

    async def me(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AuthMeResponse:
        """Endpoint to get basic user data."""
        return await self._get(
            "/auth/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthMeResponse,
        )

    async def user_token(
        self,
        *,
        user_id: str,
        expires_in: Optional[str] | Omit = omit,
        origin: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Token:
        """Use this endpoint to create a user token for a specific user.

        This token can be
        safely passed to your user-facing front-end.

        Args:
          expires_in: Token lifetime, e.g., '30m', '2h', '1d'. Defaults to 24 hours if not provided.

          origin: Origin of the request, used for CSRF protection. If set, the token will only be
              valid for requests originating from this origin.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/auth/user_token",
            body=await async_maybe_transform(
                {
                    "user_id": user_id,
                    "expires_in": expires_in,
                    "origin": origin,
                },
                auth_user_token_params.AuthUserTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Token,
        )


class AuthResourceWithRawResponse:
    def __init__(self, auth: AuthResource) -> None:
        self._auth = auth

        self.delete_user = to_raw_response_wrapper(
            auth.delete_user,
        )
        self.me = to_raw_response_wrapper(
            auth.me,
        )
        self.user_token = to_raw_response_wrapper(
            auth.user_token,
        )


class AsyncAuthResourceWithRawResponse:
    def __init__(self, auth: AsyncAuthResource) -> None:
        self._auth = auth

        self.delete_user = async_to_raw_response_wrapper(
            auth.delete_user,
        )
        self.me = async_to_raw_response_wrapper(
            auth.me,
        )
        self.user_token = async_to_raw_response_wrapper(
            auth.user_token,
        )


class AuthResourceWithStreamingResponse:
    def __init__(self, auth: AuthResource) -> None:
        self._auth = auth

        self.delete_user = to_streamed_response_wrapper(
            auth.delete_user,
        )
        self.me = to_streamed_response_wrapper(
            auth.me,
        )
        self.user_token = to_streamed_response_wrapper(
            auth.user_token,
        )


class AsyncAuthResourceWithStreamingResponse:
    def __init__(self, auth: AsyncAuthResource) -> None:
        self._auth = auth

        self.delete_user = async_to_streamed_response_wrapper(
            auth.delete_user,
        )
        self.me = async_to_streamed_response_wrapper(
            auth.me,
        )
        self.user_token = async_to_streamed_response_wrapper(
            auth.user_token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import vault_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.vault_list_response import VaultListResponse

__all__ = ["VaultsResource", "AsyncVaultsResource"]


class VaultsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> VaultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return VaultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VaultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return VaultsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[VaultListResponse]:
        """
        This endpoint lists all collections, and how many documents are in each
        collection. All documents that do not have a collection assigned are in the
        `null` collection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/vault/list",
            page=SyncCursorPage[VaultListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "size": size,
                    },
                    vault_list_params.VaultListParams,
                ),
            ),
            model=VaultListResponse,
        )


class AsyncVaultsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncVaultsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncVaultsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVaultsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncVaultsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[VaultListResponse, AsyncCursorPage[VaultListResponse]]:
        """
        This endpoint lists all collections, and how many documents are in each
        collection. All documents that do not have a collection assigned are in the
        `null` collection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/vault/list",
            page=AsyncCursorPage[VaultListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "size": size,
                    },
                    vault_list_params.VaultListParams,
                ),
            ),
            model=VaultListResponse,
        )


class VaultsResourceWithRawResponse:
    def __init__(self, vaults: VaultsResource) -> None:
        self._vaults = vaults

        self.list = to_raw_response_wrapper(
            vaults.list,
        )


class AsyncVaultsResourceWithRawResponse:
    def __init__(self, vaults: AsyncVaultsResource) -> None:
        self._vaults = vaults

        self.list = async_to_raw_response_wrapper(
            vaults.list,
        )


class VaultsResourceWithStreamingResponse:
    def __init__(self, vaults: VaultsResource) -> None:
        self._vaults = vaults

        self.list = to_streamed_response_wrapper(
            vaults.list,
        )


class AsyncVaultsResourceWithStreamingResponse:
    def __init__(self, vaults: AsyncVaultsResource) -> None:
        self._vaults = vaults

        self.list = async_to_streamed_response_wrapper(
            vaults.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from .._models import BaseModel

__all__ = ["MemoryStatusResponse"]


class MemoryStatusResponse(BaseModel):
    providers: Dict[str, Dict[str, int]]

    total: Dict[str, int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hyperspell import Hyperspell, AsyncHyperspell
from tests.utils import assert_matches_type
from hyperspell.types import (
    EvaluateScoreQueryResponse,
    EvaluateScoreHighlightResponse,
)
from hyperspell.types.shared import QueryResult

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvaluate:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_get_query(self, client: Hyperspell) -> None:
        evaluate = client.evaluate.get_query(
            "query_id",
        )
        assert_matches_type(QueryResult, evaluate, path=["response"])

    @parametrize
    def test_raw_response_get_query(self, client: Hyperspell) -> None:
        response = client.evaluate.with_raw_response.get_query(
            "query_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = response.parse()
        assert_matches_type(QueryResult, evaluate, path=["response"])

    @parametrize
    def test_streaming_response_get_query(self, client: Hyperspell) -> None:
        with client.evaluate.with_streaming_response.get_query(
            "query_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = response.parse()
            assert_matches_type(QueryResult, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get_query(self, client: Hyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `query_id` but received ''"):
            client.evaluate.with_raw_response.get_query(
                "",
            )

    @parametrize
    def test_method_score_highlight(self, client: Hyperspell) -> None:
        evaluate = client.evaluate.score_highlight(
            highlight_id="highlight_id",
        )
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    def test_method_score_highlight_with_all_params(self, client: Hyperspell) -> None:
        evaluate = client.evaluate.score_highlight(
            highlight_id="highlight_id",
            comment="comment",
            score=-1,
        )
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    def test_raw_response_score_highlight(self, client: Hyperspell) -> None:
        response = client.evaluate.with_raw_response.score_highlight(
            highlight_id="highlight_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = response.parse()
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    def test_streaming_response_score_highlight(self, client: Hyperspell) -> None:
        with client.evaluate.with_streaming_response.score_highlight(
            highlight_id="highlight_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = response.parse()
            assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_score_highlight(self, client: Hyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `highlight_id` but received ''"):
            client.evaluate.with_raw_response.score_highlight(
                highlight_id="",
            )

    @parametrize
    def test_method_score_query(self, client: Hyperspell) -> None:
        evaluate = client.evaluate.score_query(
            query_id="query_id",
        )
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    def test_method_score_query_with_all_params(self, client: Hyperspell) -> None:
        evaluate = client.evaluate.score_query(
            query_id="query_id",
            score=-1,
        )
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    def test_raw_response_score_query(self, client: Hyperspell) -> None:
        response = client.evaluate.with_raw_response.score_query(
            query_id="query_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = response.parse()
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    def test_streaming_response_score_query(self, client: Hyperspell) -> None:
        with client.evaluate.with_streaming_response.score_query(
            query_id="query_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = response.parse()
            assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_score_query(self, client: Hyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `query_id` but received ''"):
            client.evaluate.with_raw_response.score_query(
                query_id="",
            )


class TestAsyncEvaluate:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_get_query(self, async_client: AsyncHyperspell) -> None:
        evaluate = await async_client.evaluate.get_query(
            "query_id",
        )
        assert_matches_type(QueryResult, evaluate, path=["response"])

    @parametrize
    async def test_raw_response_get_query(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.evaluate.with_raw_response.get_query(
            "query_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = await response.parse()
        assert_matches_type(QueryResult, evaluate, path=["response"])

    @parametrize
    async def test_streaming_response_get_query(self, async_client: AsyncHyperspell) -> None:
        async with async_client.evaluate.with_streaming_response.get_query(
            "query_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = await response.parse()
            assert_matches_type(QueryResult, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get_query(self, async_client: AsyncHyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `query_id` but received ''"):
            await async_client.evaluate.with_raw_response.get_query(
                "",
            )

    @parametrize
    async def test_method_score_highlight(self, async_client: AsyncHyperspell) -> None:
        evaluate = await async_client.evaluate.score_highlight(
            highlight_id="highlight_id",
        )
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    async def test_method_score_highlight_with_all_params(self, async_client: AsyncHyperspell) -> None:
        evaluate = await async_client.evaluate.score_highlight(
            highlight_id="highlight_id",
            comment="comment",
            score=-1,
        )
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    async def test_raw_response_score_highlight(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.evaluate.with_raw_response.score_highlight(
            highlight_id="highlight_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = await response.parse()
        assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

    @parametrize
    async def test_streaming_response_score_highlight(self, async_client: AsyncHyperspell) -> None:
        async with async_client.evaluate.with_streaming_response.score_highlight(
            highlight_id="highlight_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = await response.parse()
            assert_matches_type(EvaluateScoreHighlightResponse, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_score_highlight(self, async_client: AsyncHyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `highlight_id` but received ''"):
            await async_client.evaluate.with_raw_response.score_highlight(
                highlight_id="",
            )

    @parametrize
    async def test_method_score_query(self, async_client: AsyncHyperspell) -> None:
        evaluate = await async_client.evaluate.score_query(
            query_id="query_id",
        )
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    async def test_method_score_query_with_all_params(self, async_client: AsyncHyperspell) -> None:
        evaluate = await async_client.evaluate.score_query(
            query_id="query_id",
            score=-1,
        )
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    async def test_raw_response_score_query(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.evaluate.with_raw_response.score_query(
            query_id="query_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluate = await response.parse()
        assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

    @parametrize
    async def test_streaming_response_score_query(self, async_client: AsyncHyperspell) -> None:
        async with async_client.evaluate.with_streaming_response.score_query(
            query_id="query_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluate = await response.parse()
            assert_matches_type(EvaluateScoreQueryResponse, evaluate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_score_query(self, async_client: AsyncHyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `query_id` but received ''"):
            await async_client.evaluate.with_raw_response.score_query(
                query_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ConnectionListResponse", "Connection"]


class Connection(BaseModel):
    id: str
    """The connection's id"""

    integration_id: str
    """The connection's integration id"""

    label: Optional[str] = None
    """The connection's label"""

    provider: Literal[
        "collections",
        "vault",
        "web_crawler",
        "notion",
        "slack",
        "google_calendar",
        "reddit",
        "box",
        "google_drive",
        "airtable",
        "algolia",
        "amplitude",
        "asana",
        "ashby",
        "bamboohr",
        "basecamp",
        "bubbles",
        "calendly",
        "confluence",
        "clickup",
        "datadog",
        "deel",
        "discord",
        "dropbox",
        "exa",
        "facebook",
        "front",
        "github",
        "gitlab",
        "google_docs",
        "google_mail",
        "google_sheet",
        "hubspot",
        "jira",
        "linear",
        "microsoft_teams",
        "mixpanel",
        "monday",
        "outlook",
        "perplexity",
        "rippling",
        "salesforce",
        "segment",
        "todoist",
        "twitter",
        "zoom",
    ]
    """The connection's provider"""


class ConnectionListResponse(BaseModel):
    connections: List[Connection]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from .._models import BaseModel

__all__ = ["Token"]


class Token(BaseModel):
    token: str

    expires_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .calendar import Calendar as Calendar
from .slack_list_params import SlackListParams as SlackListParams
from .web_crawler_index_params import WebCrawlerIndexParams as WebCrawlerIndexParams
from .web_crawler_index_response import WebCrawlerIndexResponse as WebCrawlerIndexResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["MemoryUploadParams"]


class MemoryUploadParams(TypedDict, total=False):
    file: Required[FileTypes]
    """The file to ingest."""

    collection: Optional[str]
    """The collection to add the document to."""

    metadata: Optional[str]
    """Custom metadata as JSON string for filtering.

    Keys must be alphanumeric with underscores, max 64 chars. Values must be string,
    number, or boolean.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hyperspell import Hyperspell, AsyncHyperspell
from tests.utils import assert_matches_type
from hyperspell.types import Token, AuthMeResponse, AuthDeleteUserResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuth:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_delete_user(self, client: Hyperspell) -> None:
        auth = client.auth.delete_user()
        assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

    @parametrize
    def test_raw_response_delete_user(self, client: Hyperspell) -> None:
        response = client.auth.with_raw_response.delete_user()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = response.parse()
        assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

    @parametrize
    def test_streaming_response_delete_user(self, client: Hyperspell) -> None:
        with client.auth.with_streaming_response.delete_user() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = response.parse()
            assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_me(self, client: Hyperspell) -> None:
        auth = client.auth.me()
        assert_matches_type(AuthMeResponse, auth, path=["response"])

    @parametrize
    def test_raw_response_me(self, client: Hyperspell) -> None:
        response = client.auth.with_raw_response.me()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = response.parse()
        assert_matches_type(AuthMeResponse, auth, path=["response"])

    @parametrize
    def test_streaming_response_me(self, client: Hyperspell) -> None:
        with client.auth.with_streaming_response.me() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = response.parse()
            assert_matches_type(AuthMeResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_user_token(self, client: Hyperspell) -> None:
        auth = client.auth.user_token(
            user_id="user_id",
        )
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    def test_method_user_token_with_all_params(self, client: Hyperspell) -> None:
        auth = client.auth.user_token(
            user_id="user_id",
            expires_in="30m",
            origin="origin",
        )
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    def test_raw_response_user_token(self, client: Hyperspell) -> None:
        response = client.auth.with_raw_response.user_token(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = response.parse()
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    def test_streaming_response_user_token(self, client: Hyperspell) -> None:
        with client.auth.with_streaming_response.user_token(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = response.parse()
            assert_matches_type(Token, auth, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAuth:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_delete_user(self, async_client: AsyncHyperspell) -> None:
        auth = await async_client.auth.delete_user()
        assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

    @parametrize
    async def test_raw_response_delete_user(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.auth.with_raw_response.delete_user()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = await response.parse()
        assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

    @parametrize
    async def test_streaming_response_delete_user(self, async_client: AsyncHyperspell) -> None:
        async with async_client.auth.with_streaming_response.delete_user() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = await response.parse()
            assert_matches_type(AuthDeleteUserResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_me(self, async_client: AsyncHyperspell) -> None:
        auth = await async_client.auth.me()
        assert_matches_type(AuthMeResponse, auth, path=["response"])

    @parametrize
    async def test_raw_response_me(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.auth.with_raw_response.me()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = await response.parse()
        assert_matches_type(AuthMeResponse, auth, path=["response"])

    @parametrize
    async def test_streaming_response_me(self, async_client: AsyncHyperspell) -> None:
        async with async_client.auth.with_streaming_response.me() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = await response.parse()
            assert_matches_type(AuthMeResponse, auth, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_user_token(self, async_client: AsyncHyperspell) -> None:
        auth = await async_client.auth.user_token(
            user_id="user_id",
        )
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    async def test_method_user_token_with_all_params(self, async_client: AsyncHyperspell) -> None:
        auth = await async_client.auth.user_token(
            user_id="user_id",
            expires_in="30m",
            origin="origin",
        )
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    async def test_raw_response_user_token(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.auth.with_raw_response.user_token(
            user_id="user_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        auth = await response.parse()
        assert_matches_type(Token, auth, path=["response"])

    @parametrize
    async def test_streaming_response_user_token(self, async_client: AsyncHyperspell) -> None:
        async with async_client.auth.with_streaming_response.user_token(
            user_id="user_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            auth = await response.parse()
            assert_matches_type(Token, auth, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hyperspell import Hyperspell, AsyncHyperspell
from tests.utils import assert_matches_type
from hyperspell.types import IntegrationListResponse, IntegrationConnectResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIntegrations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: Hyperspell) -> None:
        integration = client.integrations.list()
        assert_matches_type(IntegrationListResponse, integration, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Hyperspell) -> None:
        response = client.integrations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration = response.parse()
        assert_matches_type(IntegrationListResponse, integration, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Hyperspell) -> None:
        with client.integrations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration = response.parse()
            assert_matches_type(IntegrationListResponse, integration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_connect(self, client: Hyperspell) -> None:
        integration = client.integrations.connect(
            integration_id="integration_id",
        )
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    def test_method_connect_with_all_params(self, client: Hyperspell) -> None:
        integration = client.integrations.connect(
            integration_id="integration_id",
            redirect_url="redirect_url",
        )
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    def test_raw_response_connect(self, client: Hyperspell) -> None:
        response = client.integrations.with_raw_response.connect(
            integration_id="integration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration = response.parse()
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    def test_streaming_response_connect(self, client: Hyperspell) -> None:
        with client.integrations.with_streaming_response.connect(
            integration_id="integration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration = response.parse()
            assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_connect(self, client: Hyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `integration_id` but received ''"):
            client.integrations.with_raw_response.connect(
                integration_id="",
            )


class TestAsyncIntegrations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_list(self, async_client: AsyncHyperspell) -> None:
        integration = await async_client.integrations.list()
        assert_matches_type(IntegrationListResponse, integration, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.integrations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration = await response.parse()
        assert_matches_type(IntegrationListResponse, integration, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHyperspell) -> None:
        async with async_client.integrations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration = await response.parse()
            assert_matches_type(IntegrationListResponse, integration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_connect(self, async_client: AsyncHyperspell) -> None:
        integration = await async_client.integrations.connect(
            integration_id="integration_id",
        )
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    async def test_method_connect_with_all_params(self, async_client: AsyncHyperspell) -> None:
        integration = await async_client.integrations.connect(
            integration_id="integration_id",
            redirect_url="redirect_url",
        )
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    async def test_raw_response_connect(self, async_client: AsyncHyperspell) -> None:
        response = await async_client.integrations.with_raw_response.connect(
            integration_id="integration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        integration = await response.parse()
        assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

    @parametrize
    async def test_streaming_response_connect(self, async_client: AsyncHyperspell) -> None:
        async with async_client.integrations.with_streaming_response.connect(
            integration_id="integration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            integration = await response.parse()
            assert_matches_type(IntegrationConnectResponse, integration, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_connect(self, async_client: AsyncHyperspell) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `integration_id` but received ''"):
            await async_client.integrations.with_raw_response.connect(
                integration_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.integrations import web_crawler_index_params
from ...types.integrations.web_crawler_index_response import WebCrawlerIndexResponse

__all__ = ["WebCrawlerResource", "AsyncWebCrawlerResource"]


class WebCrawlerResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> WebCrawlerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return WebCrawlerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WebCrawlerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return WebCrawlerResourceWithStreamingResponse(self)

    def index(
        self,
        *,
        url: str,
        limit: int | Omit = omit,
        max_depth: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerIndexResponse:
        """
        Recursively crawl a website to make it available for indexed search.

        Args:
          url: The base URL of the website to crawl

          limit: Maximum number of pages to crawl in total

          max_depth: Maximum depth of links to follow during crawling

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/integrations/web_crawler/index",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "url": url,
                        "limit": limit,
                        "max_depth": max_depth,
                    },
                    web_crawler_index_params.WebCrawlerIndexParams,
                ),
            ),
            cast_to=WebCrawlerIndexResponse,
        )


class AsyncWebCrawlerResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncWebCrawlerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/hyperspell/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncWebCrawlerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWebCrawlerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/hyperspell/python-sdk#with_streaming_response
        """
        return AsyncWebCrawlerResourceWithStreamingResponse(self)

    async def index(
        self,
        *,
        url: str,
        limit: int | Omit = omit,
        max_depth: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> WebCrawlerIndexResponse:
        """
        Recursively crawl a website to make it available for indexed search.

        Args:
          url: The base URL of the website to crawl

          limit: Maximum number of pages to crawl in total

          max_depth: Maximum depth of links to follow during crawling

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/integrations/web_crawler/index",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "url": url,
                        "limit": limit,
                        "max_depth": max_depth,
                    },
                    web_crawler_index_params.WebCrawlerIndexParams,
                ),
            ),
            cast_to=WebCrawlerIndexResponse,
        )


class WebCrawlerResourceWithRawResponse:
    def __init__(self, web_crawler: WebCrawlerResource) -> None:
        self._web_crawler = web_crawler

        self.index = to_raw_response_wrapper(
            web_crawler.index,
        )


class AsyncWebCrawlerResourceWithRawResponse:
    def __init__(self, web_crawler: AsyncWebCrawlerResource) -> None:
        self._web_crawler = web_crawler

        self.index = async_to_raw_response_wrapper(
            web_crawler.index,
        )


class WebCrawlerResourceWithStreamingResponse:
    def __init__(self, web_crawler: WebCrawlerResource) -> None:
        self._web_crawler = web_crawler

        self.index = to_streamed_response_wrapper(
            web_crawler.index,
        )


class AsyncWebCrawlerResourceWithStreamingResponse:
    def __init__(self, web_crawler: AsyncWebCrawlerResource) -> None:
        self._web_crawler = web_crawler

        self.index = async_to_streamed_response_wrapper(
            web_crawler.index,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = [
    "MemorySearchParams",
    "Options",
    "OptionsBox",
    "OptionsCollections",
    "OptionsGoogleCalendar",
    "OptionsGoogleDrive",
    "OptionsGoogleMail",
    "OptionsNotion",
    "OptionsReddit",
    "OptionsSlack",
    "OptionsWebCrawler",
]


class MemorySearchParams(TypedDict, total=False):
    query: Required[str]
    """Query to run."""

    answer: bool
    """If true, the query will be answered along with matching source documents."""

    max_results: int
    """Maximum number of results to return."""

    options: Options
    """Search options for the query."""

    sources: List[
        Literal[
            "collections",
            "vault",
            "web_crawler",
            "notion",
            "slack",
            "google_calendar",
            "reddit",
            "box",
            "google_drive",
            "airtable",
            "algolia",
            "amplitude",
            "asana",
            "ashby",
            "bamboohr",
            "basecamp",
            "bubbles",
            "calendly",
            "confluence",
            "clickup",
            "datadog",
            "deel",
            "discord",
            "dropbox",
            "exa",
            "facebook",
            "front",
            "github",
            "gitlab",
            "google_docs",
            "google_mail",
            "google_sheet",
            "hubspot",
            "jira",
            "linear",
            "microsoft_teams",
            "mixpanel",
            "monday",
            "outlook",
            "perplexity",
            "rippling",
            "salesforce",
            "segment",
            "todoist",
            "twitter",
            "zoom",
        ]
    ]
    """Only query documents from these sources."""


class OptionsBox(TypedDict, total=False):
    """Search options for Box"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsCollections(TypedDict, total=False):
    """Search options for vault"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsGoogleCalendar(TypedDict, total=False):
    """Search options for Google Calendar"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    calendar_id: Optional[str]
    """The ID of the calendar to search.

    If not provided, it will use the ID of the default calendar. You can get the
    list of calendars with the `/integrations/google_calendar/list` endpoint.
    """

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsGoogleDrive(TypedDict, total=False):
    """Search options for Google Drive"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsGoogleMail(TypedDict, total=False):
    """Search options for Gmail"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    label_ids: SequenceNotStr[str]
    """List of label IDs to filter messages (e.g., ['INBOX', 'SENT', 'DRAFT']).

    Multiple labels are combined with OR logic - messages matching ANY specified
    label will be returned. If empty, no label filtering is applied (searches all
    accessible messages).
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsNotion(TypedDict, total=False):
    """Search options for Notion"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    notion_page_ids: SequenceNotStr[str]
    """List of Notion page IDs to search.

    If not provided, all pages in the workspace will be searched.
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsReddit(TypedDict, total=False):
    """Search options for Reddit"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    period: Literal["hour", "day", "week", "month", "year", "all"]
    """The time period to search. Defaults to 'month'."""

    sort: Literal["relevance", "new", "hot", "top", "comments"]
    """The sort order of the posts. Defaults to 'relevance'."""

    subreddit: Optional[str]
    """The subreddit to search.

    If not provided, the query will be searched for in all subreddits.
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsSlack(TypedDict, total=False):
    """Search options for Slack"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    channels: SequenceNotStr[str]
    """List of Slack channels to include (by id, name, or #name)."""

    exclude_archived: Optional[bool]
    """If set, pass 'exclude_archived' to Slack. If None, omit the param."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    include_dms: bool
    """Include direct messages (im) when listing conversations."""

    include_group_dms: bool
    """Include group DMs (mpim) when listing conversations."""

    include_private: bool
    """Include private channels when constructing Slack 'types'.

    Defaults to False to preserve existing cassette query params.
    """

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class OptionsWebCrawler(TypedDict, total=False):
    """Search options for Web Crawler"""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    max_depth: int
    """Maximum depth to crawl from the starting URL"""

    url: Optional[str]
    """The URL to crawl"""

    weight: float
    """Weight of results from this source.

    A weight greater than 1.0 means more results from this source will be returned,
    a weight less than 1.0 means fewer results will be returned. This will only
    affect results if multiple sources are queried at the same time.
    """


class Options(TypedDict, total=False):
    """Search options for the query."""

    after: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created on or after this date."""

    answer_model: Literal["llama-3.1", "gemma2", "qwen-qwq", "mistral-saba", "llama-4-scout", "deepseek-r1"]
    """Model to use for answer generation when answer=True"""

    before: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Only query documents created before this date."""

    box: OptionsBox
    """Search options for Box"""

    collections: OptionsCollections
    """Search options for vault"""

    filter: Optional[Dict[str, object]]
    """Metadata filters using MongoDB-style operators.

    Example: {'status': 'published', 'priority': {'$gt': 3}}
    """

    google_calendar: OptionsGoogleCalendar
    """Search options for Google Calendar"""

    google_drive: OptionsGoogleDrive
    """Search options for Google Drive"""

    google_mail: OptionsGoogleMail
    """Search options for Gmail"""

    max_results: int
    """Maximum number of results to return."""

    notion: OptionsNotion
    """Search options for Notion"""

    reddit: OptionsReddit
    """Search options for Reddit"""

    slack: OptionsSlack
    """Search options for Slack"""

    web_crawler: OptionsWebCrawler
    """Search options for Web Crawler"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MemoryAddParams"]


class MemoryAddParams(TypedDict, total=False):
    text: Required[str]
    """Full text of the document."""

    collection: Optional[str]
    """The collection to add the document to for easier retrieval."""

    date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Date of the document.

    Depending on the document, this could be the creation date or date the document
    was last updated (eg. for a chat transcript, this would be the date of the last
    message). This helps the ranking algorithm and allows you to filter by date
    range.
    """

    metadata: Optional[Dict[str, Union[str, float, bool]]]
    """Custom metadata for filtering.

    Keys must be alphanumeric with underscores, max 64 chars. Values must be string,
    number, or boolean.
    """

    resource_id: str
    """The resource ID to add the document to.

    If not provided, a new resource ID will be generated. If provided, the document
    will be updated if it already exists.
    """

    title: Optional[str]
    """Title of the document."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["Calendar", "Item"]


class Item(BaseModel):
    id: str
    """The ID of the calendar"""

    name: str
    """The name of the calendar"""

    primary: bool
    """Whether the calendar is the primary calendar of the user"""

    timezone: Optional[str] = None
    """Default timezone of the calendar"""


class Calendar(BaseModel):
    items: List[Item]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["EvaluateScoreQueryResponse"]


class EvaluateScoreQueryResponse(BaseModel):
    message: str
    """A message describing the result."""

    success: bool
    """Whether the feedback was successfully saved."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["EvaluateScoreQueryParams"]


class EvaluateScoreQueryParams(TypedDict, total=False):
    score: float
    """Rating of the query result from -1 (bad) to +1 (good)."""
